// Functions for easy use of HD44780 LCD display on Kamami KAMeLeon STM32L496 board.
// jk 12'2017
// The initialization code was generated in STM32CubeMX.

#include "kamami_l496_lcd.h"
#include "stm32l4xx_hal.h"

#define LCD_RS_Pin GPIO_PIN_2
#define LCD_RS_GPIO_Port GPIOF
#define LCD_D4_Pin GPIO_PIN_7
#define LCD_D4_GPIO_Port GPIOE
#define LCD_D5_Pin GPIO_PIN_8
#define LCD_D5_GPIO_Port GPIOE
#define LCD_D6_Pin GPIO_PIN_9
#define LCD_D6_GPIO_Port GPIOE
#define LCD_D7_Pin GPIO_PIN_10
#define LCD_D7_GPIO_Port GPIOE
#define LCD_BKL_Pin GPIO_PIN_13
#define LCD_BKL_GPIO_Port GPIOB
#define LCD_D0_Pin GPIO_PIN_14
#define LCD_D0_GPIO_Port GPIOD
#define LCD_D1_Pin GPIO_PIN_15
#define LCD_D1_GPIO_Port GPIOD
#define LCD_D2_Pin GPIO_PIN_0
#define LCD_D2_GPIO_Port GPIOD
#define LCD_D3_Pin GPIO_PIN_1
#define LCD_D3_GPIO_Port GPIOD
#define LCD_RW_Pin GPIO_PIN_4
#define LCD_RW_GPIO_Port GPIOD
#define LCD_E_Pin GPIO_PIN_5
#define LCD_E_GPIO_Port GPIOD

#define LCD_DATA_GPIOD_MSK	(LCD_D0_Pin | LCD_D1_Pin | LCD_D2_Pin | LCD_D3_Pin)
#define LCD_DATA_GPIOE_MSK	(LCD_D4_Pin | LCD_D5_Pin | LCD_D6_Pin | LCD_D7_Pin)

// as the LCD data lines are not all on the same port nor are they in order, 
// the table below provides mapping from data bit number to the port and pin it is connected to
struct lcd_def_ {
	GPIO_TypeDef* port;
	uint16_t pin;
};

struct lcd_def_ lcd_defs[8] = {
	LCD_D0_GPIO_Port, LCD_D0_Pin,
	LCD_D1_GPIO_Port, LCD_D1_Pin,
	LCD_D2_GPIO_Port, LCD_D2_Pin,
	LCD_D3_GPIO_Port, LCD_D3_Pin,
	LCD_D4_GPIO_Port, LCD_D4_Pin,
	LCD_D5_GPIO_Port, LCD_D5_Pin,
	LCD_D6_GPIO_Port, LCD_D6_Pin,
	LCD_D7_GPIO_Port, LCD_D7_Pin,	
};

static void lcd_gpio_init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOF_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LCD_RS_GPIO_Port, LCD_RS_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOE, LCD_D4_Pin|LCD_D5_Pin|LCD_D6_Pin|LCD_D7_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LCD_BKL_GPIO_Port, LCD_BKL_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOD, LCD_D0_Pin|LCD_D1_Pin|LCD_D2_Pin|LCD_D3_Pin 
                          |LCD_RW_Pin|LCD_E_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin : LCD_RS_Pin */
  GPIO_InitStruct.Pin = LCD_RS_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LCD_RS_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : LCD_D4_Pin LCD_D5_Pin LCD_D6_Pin LCD_D7_Pin */
  GPIO_InitStruct.Pin = LCD_D4_Pin|LCD_D5_Pin|LCD_D6_Pin|LCD_D7_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  /*Configure GPIO pin : LCD_BKL_Pin */
  GPIO_InitStruct.Pin = LCD_BKL_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LCD_BKL_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : LCD_D0_Pin LCD_D1_Pin LCD_D2_Pin LCD_D3_Pin 
                           LCD_RW_Pin LCD_E_Pin */
  GPIO_InitStruct.Pin = LCD_D0_Pin|LCD_D1_Pin|LCD_D2_Pin|LCD_D3_Pin 
                          |LCD_RW_Pin|LCD_E_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);
}


/** Activate Power Pin that supplies LCD module */
void lcd_backlight_on (void)
{
	HAL_GPIO_WritePin(LCD_BKL_GPIO_Port, LCD_BKL_Pin, GPIO_PIN_SET);
}

/* Disable Power Pin that supplies LCD module */
void lcd_backlight_off (void)
{
	HAL_GPIO_WritePin(LCD_BKL_GPIO_Port, LCD_BKL_Pin, GPIO_PIN_RESET);
}

/* Activate Enable Pin from LCD module  */
static void lcd_enable (void)
{
	HAL_GPIO_WritePin(LCD_E_GPIO_Port, LCD_E_Pin, GPIO_PIN_SET);
	HAL_Delay(2);
	HAL_GPIO_WritePin(LCD_E_GPIO_Port, LCD_E_Pin, GPIO_PIN_RESET);
}

static void lcd_8bit(uint8_t d)
{
	for(uint32_t i = 0; i < 8; ++i)
	{
		HAL_GPIO_WritePin(lcd_defs[i].port, lcd_defs[i].pin, (d >> i) & 1 ? GPIO_PIN_SET : GPIO_PIN_RESET);
	}
	
	lcd_enable();
	HAL_Delay(2);	
}

/* Send command to LCD module */
static void lcd_cmd(unsigned char cmd)
{
	HAL_GPIO_WritePin(LCD_RS_GPIO_Port, LCD_RS_Pin, GPIO_PIN_RESET);
	lcd_8bit(cmd);
}

/* Send data to LCD module */
static void lcd_data(uint8_t data)
{
	HAL_GPIO_WritePin(LCD_RS_GPIO_Port, LCD_RS_Pin, GPIO_PIN_SET);
	lcd_8bit(data);
}

/* Clear LCD module display */
void lcd_clear(void)
{
  lcd_cmd(0x01);
  HAL_Delay(2);
}

/* Initializes HD44780 LCD module in 4-bit mode */
void lcd_init(void)
{
	lcd_gpio_init();
		//HAL_Delay(500);
	HAL_GPIO_WritePin(LCD_E_GPIO_Port, LCD_E_Pin, GPIO_PIN_SET);
	HAL_GPIO_WritePin(LCD_RS_GPIO_Port, LCD_RS_Pin, GPIO_PIN_RESET);

	//8-bit mode
	lcd_cmd(0x38);
	//Display On
	lcd_cmd(0x0C);
	//Entry mode set
	lcd_cmd(0x06);
	lcd_clear();
	HAL_Delay(200);
}

/* Set Cursor to a specified location given by row and column information
  * @param  Row Number (1 to 2)
  * @param  Column Number (1 to 16) Assuming a 2 X 16 characters display */
static void lcd_set_position(uint8_t row, uint8_t column)
{
  column--;
  switch (row)
  {
    case 1:
      /* Set cursor to 1st row address and add index*/
      lcd_cmd(column |= 0x80);
      break;
    case 2:
      /* Set cursor to 2nd row address and add index*/
      lcd_cmd(column |= 0x40 | 0x80);
      break;
    default:
      break;
  }
}

/* Display a characters string */
void lcd_display(char *data, uint8_t len, uint8_t row, uint8_t col)
{	
	lcd_set_position(row, col);
	while (len-- > 0)
	{
	lcd_data(*data++);
	}
}


