// Functions for easy use of LEDs (LED0..7) on Kamami KAMeLeon STM32L496 board.
// jk 11'2017

#ifndef	_KAMAMI_L496_LED_H
#define	_KAMAMI_L496_LED_H
#include <stdint.h>

/* Initialize LEDs (LED0..7). 
 * The function should be called before using any other LED function. 
 * It initializes GPIO lines. */
void led_init(void);


/* Turn on the LED of a given number. 
 * @param led_num: number of LED to turn on */ 
void led_on(uint8_t led_num);

/* Turn off the LED of a given number. 
 * @param led_num: number of LED to turn off */ 
void led_off(uint8_t led_num);

/* Toggle the LED of a given number. 
 * @param led_num: number of LED to toggle */ 
void led_toggle(uint8_t led_num);


// In all the functions below bit number x corresponds to LEDx, 
// e.g. giving a bitmask of 0x43 will affect LEDs 0, 1 and 6.

/* Set the state of all 8 LEDs. 
 * @param led_msk: bitmask giving the state of LEDs; bit of value 0 turns the 
 *                 correstponding LED off, bit of value 1 - on  */ 
void led_set_mul(uint8_t led_msk);

/* Turn on multiple LEDs given by a bitmask. 
 * @param led_msk: bitmask of LEDs to turn on */ 
void led_on_mul(uint8_t led_msk);

/* Turn off multiple LEDs given by a bitmask. 
 * @param led_msk: bitmask of LEDs to turn off */ 
void led_off_mul(uint8_t led_msk);

/* Toggle multiple LEDs given by a bitmask. 
 * @param led_msk: bitmask of LEDs to toggle */ 
void led_toggle_mul(uint8_t led_msk);

#endif
