// Functions for easy use of QSPI memory on Kamami KAMeLeon STM32L496 board.
// jk 12'2017
// The initialization code was generated in STM32CubeMX.

#include "kamami_l496_qspi.h"
#include "stm32l4xx_hal.h"

/* Read Operations */
#define READ_CMD                             0x03
#define FAST_READ_CMD                        0x0B
#define DUAL_OUT_READ_CMD                    0x3B
#define DUAL_INOUT_READ_CMD                  0xBB
#define QUAD_OUT_READ_CMD                    0x6B
#define QUAD_INOUT_READ_CMD                  0xEB

/* Read Operations with 4b */
#define READ_CMD                             0x03
#define FAST_READ_CMD                        0x0B
#define DUAL_OUT_READ_CMD                    0x3B
#define DUAL_INOUT_READ_CMD                  0xBB
#define QUAD_OUT_READ_CMD                    0x6B
#define QUAD_INOUT_READ_CMD                  0xEB

/* Program Operations */
#define PAGE_PROG_CMD                        0x02
#define QUAD_PAGE_PROG_CMD                   0x32

/* Erase Operations */
#define SUBBLOCK32K_ERASE_CMD                0x52
#define SUBBLOCK4K_ERASE_CMD                 0x20
#define SECTOR_ERASE_CMD                     0xD8
#define CHIP_ERASE_CMD                       0x60
#define CHIP_ERASE_CMD_2                     0xC7

/* Identification Operations */
#define READ_ID_CMD                          0x9F

/* Write Operations */
#define WRITE_ENABLE_CMD                     0x06
#define WRITE_DISABLE_CMD                    0x04

/* Register Operations */
#define READ_STATUS_REG_CMD                  0x05
#define READ_STATUS_REG2_CMD                 0x35
#define READ_FLAG_STATUS_REG_CMD             0x70
#define READ_NONVOL_CFG_REG_CMD              0xB5
#define READ_VOL_CFG_REG_CMD	             0x85
#define READ_ENHVOL_CFG_REG_CMD              0x65
#define READ_EXT_ADDR_REG_CMD     	         0xC8
#define WRITE_STATUS_REG_CMD                 0x01
#define WRITE_STATUS_REG2_CMD                0x31
#define CLEAR_FLAG_STATUS_REG_CMD            0x50
#define WRITE_NONVOL_CFG_REG_CMD             0xB1
#define WRITE_VOL_CFG_REG_CMD	             0x81
#define WRITE_ENHVOL_CFG_REG_CMD             0x61
#define WRITE_EXT_ADDR_REG_CMD     	         0xC5

#define READ_SEC_REG_CMD                     0x2B
#define WRITE_SEC_REG_CMD                    0x2F

/* Burst Operations */
#define SET_BURST_LENGTH_CMD                 0xC0

/* One-Time Programmable Operations */
#define ENTER_SECURED_OTP_CMD                0xB1
#define EXIT_SECURED_OTP_CMD                 0xC1

/* Reset Operations */
#define RESET_ENABLE_CMD                     0x66
#define RESET_MEMORY_CMD                     0x99

/* Default dummy clocks cycles */
#define DUMMY_CLOCK_CYCLES_READ              8
#define DUMMY_CLOCK_CYCLES_READ_QUAD         4

/* End address of the QSPI memory */
#define QSPI_END_ADDR              (1 << QSPI_FLASH_SIZE)

QSPI_HandleTypeDef hqspi;

static void MX_QUADSPI_Init(void);

static void qspi_write_enable(void)
{
  QSPI_CommandTypeDef     qspicmd;
	
  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
  qspicmd.Instruction       = WRITE_ENABLE_CMD;
  qspicmd.AddressMode       = QSPI_ADDRESS_NONE;
  qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
  qspicmd.DataMode          = QSPI_DATA_NONE;
  qspicmd.DummyCycles       = 0;
  qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
  qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
  qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
  
  QSPI_AutoPollingTypeDef autopoll;
  autopoll.Match           = 0x02;
  autopoll.Mask            = 0x02;
  autopoll.MatchMode       = QSPI_MATCH_MODE_AND;
  autopoll.StatusBytesSize = 1;
  autopoll.Interval        = 0x10;
  autopoll.AutomaticStop   = QSPI_AUTOMATIC_STOP_ENABLE;
  qspicmd.Instruction    = READ_STATUS_REG_CMD;
  qspicmd.DataMode       = QSPI_DATA_1_LINE;

  HAL_QSPI_AutoPolling(&hqspi, &qspicmd, &autopoll, 1000);
}

void qspi_read_id(uint8_t* buffer)
{
  QSPI_CommandTypeDef     qspicmd;
	
  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
  qspicmd.Instruction       = READ_ID_CMD;
  qspicmd.AddressMode       = QSPI_ADDRESS_NONE;
  qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
  qspicmd.DataMode          = QSPI_DATA_1_LINE;
  qspicmd.NbData      		= 2;
  qspicmd.DummyCycles       = 0;
  qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
  qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
  qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
  
  HAL_QSPI_Receive(&hqspi, buffer, 1000);
}

void qspi_read(uint32_t address, uint8_t* buffer, uint32_t length)
{
  QSPI_CommandTypeDef     qspicmd;
	
  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
  qspicmd.Instruction       = QUAD_INOUT_READ_CMD;
  qspicmd.AddressMode       = QSPI_ADDRESS_4_LINES;
  qspicmd.AddressSize       = QSPI_ADDRESS_24_BITS;
  qspicmd.Address     		= address;
  qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_4_LINES;
  qspicmd.AlternateBytesSize = 1;
  qspicmd.AlternateBytes = 0;
  qspicmd.DataMode          = QSPI_DATA_4_LINES;
  qspicmd.NbData      		= length;
  qspicmd.DummyCycles       = DUMMY_CLOCK_CYCLES_READ_QUAD;
  qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
  qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
  qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
  
  HAL_QSPI_Receive(&hqspi, buffer, 1000);
}

void qspi_erase_chip()
{
  qspi_write_enable();	
	
  QSPI_CommandTypeDef     qspicmd;
	
  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
  qspicmd.Instruction       = CHIP_ERASE_CMD;
  qspicmd.AddressMode       = QSPI_ADDRESS_NONE;
  qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
  qspicmd.DataMode          = QSPI_DATA_NONE;
  qspicmd.DummyCycles       = 0;
  qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
  qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
  qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
}

void qspi_erase_sector(uint32_t address)
{
  qspi_write_enable();	
	
  QSPI_CommandTypeDef     qspicmd;
	
  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
  qspicmd.Instruction       = SECTOR_ERASE_CMD ;
  qspicmd.AddressMode       = QSPI_ADDRESS_1_LINE;
  qspicmd.AddressSize       = QSPI_ADDRESS_24_BITS;
  qspicmd.Address     		= address;
  qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
  qspicmd.DataMode          = QSPI_DATA_NONE;
  qspicmd.DummyCycles       = 0;
  qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
  qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
  qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
}

static uint8_t QSPI_AutoPollingMemReady(void)
{
  QSPI_CommandTypeDef     qspicmd;
  QSPI_AutoPollingTypeDef autopoll;

  /* Configure automatic polling mode to wait for memory ready */  
  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
  qspicmd.Instruction       = READ_STATUS_REG_CMD;
  qspicmd.AddressMode       = QSPI_ADDRESS_NONE;
  qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
  qspicmd.DataMode          = QSPI_DATA_1_LINE;
  qspicmd.DummyCycles       = 0;
  qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
  qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
  qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

  autopoll.Match           = 0;
  autopoll.Mask            = 1;
  autopoll.MatchMode       = QSPI_MATCH_MODE_AND;
  autopoll.StatusBytesSize = 1;
  autopoll.Interval        = 0x10;
  autopoll.AutomaticStop   = QSPI_AUTOMATIC_STOP_ENABLE;

  HAL_QSPI_AutoPolling(&hqspi, &qspicmd, &autopoll, 1000);

  return 0;
}

#define QSPI_PAGE_MASK (~(QSPI_PAGE_SIZE - 1))

void qspi_write(uint32_t address, uint8_t* buffer, uint32_t length)
{
		QSPI_CommandTypeDef qspicmd;
	  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
		qspicmd.Instruction       = QUAD_PAGE_PROG_CMD;
		qspicmd.AddressMode       = QSPI_ADDRESS_1_LINE;
		qspicmd.AddressSize       = QSPI_ADDRESS_24_BITS;
		qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
		qspicmd.DataMode          = QSPI_DATA_4_LINES;
		qspicmd.DummyCycles       = 0;
		qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
		qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
		qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

		uint32_t len;	
		while (length)
		{
				len = length;
				if (((address + len - 1) ^ address) & QSPI_PAGE_MASK)	// more than one page to write
					len = ((address + QSPI_PAGE_SIZE) & QSPI_PAGE_MASK) - address;
				
				qspicmd.Address = address;
				qspicmd.NbData  = len;

				qspi_write_enable();				
				HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
				HAL_QSPI_Transmit(&hqspi, buffer, HAL_QPSI_TIMEOUT_DEFAULT_VALUE); 
				QSPI_AutoPollingMemReady();

				address += len;
				buffer += len;
				length -= len;
		}
}

static void qspi_quadmode()
{
  QSPI_CommandTypeDef qspicmd;
  uint8_t reg[2];

  /* Read status register */
  qspicmd.InstructionMode   = QSPI_INSTRUCTION_1_LINE;
  qspicmd.Instruction       = READ_STATUS_REG_CMD;
  qspicmd.AddressMode       = QSPI_ADDRESS_NONE;
  qspicmd.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
  qspicmd.DataMode          = QSPI_DATA_1_LINE;
  qspicmd.DummyCycles       = 0;
  qspicmd.NbData            = 1;
  qspicmd.DdrMode           = QSPI_DDR_MODE_DISABLE;
  qspicmd.DdrHoldHalfCycle  = QSPI_DDR_HHC_ANALOG_DELAY;
  qspicmd.SIOOMode          = QSPI_SIOO_INST_EVERY_CMD;

  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
  HAL_QSPI_Receive(&hqspi, &reg[0], 1000);

	qspicmd.Instruction       = READ_STATUS_REG2_CMD;
  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
  HAL_QSPI_Receive(&hqspi, &reg[1], 1000);
	
  /* Enable write operations */
  qspi_write_enable();
	
  reg[1] |= 2; // set Quad Enable bit
  qspicmd.Instruction = WRITE_STATUS_REG_CMD;
  qspicmd.NbData            = 2;
  
  HAL_QSPI_Command(&hqspi, &qspicmd, 1000);
  HAL_QSPI_Transmit(&hqspi, reg, 1000);

  /* Wait that memory is ready */  

  /* Configure automatic polling mode to wait for memory ready ------ */  
  qspicmd.Instruction    = READ_STATUS_REG_CMD;

  QSPI_AutoPollingTypeDef autopoll;
  autopoll.Match           = 0x00;
  autopoll.Mask            = 0x01;
  autopoll.MatchMode       = QSPI_MATCH_MODE_AND;
  autopoll.StatusBytesSize = 1;
  autopoll.Interval        = 0x10;
  autopoll.AutomaticStop   = QSPI_AUTOMATIC_STOP_ENABLE;

  HAL_QSPI_AutoPolling(&hqspi, &qspicmd, &autopoll, 1000);
	
  GPIO_InitTypeDef GPIO_InitStruct;
	GPIO_InitStruct.Pin = GPIO_PIN_6|GPIO_PIN_7;
	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
	GPIO_InitStruct.Alternate = GPIO_AF10_QUADSPI;
	HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);	
}

void qspi_init(void)
{	
	__HAL_RCC_GPIOE_CLK_ENABLE();
  __HAL_RCC_GPIOF_CLK_ENABLE();
  MX_QUADSPI_Init();
  qspi_quadmode();
}

/* QUADSPI init function */
static void MX_QUADSPI_Init(void)
{
  hqspi.Instance = QUADSPI;
  hqspi.Init.ClockPrescaler = 1;
  hqspi.Init.FifoThreshold = 4;
  hqspi.Init.SampleShifting = QSPI_SAMPLE_SHIFTING_NONE;
  hqspi.Init.FlashSize = QSPI_FLASH_SIZE;
  hqspi.Init.ChipSelectHighTime = QSPI_CS_HIGH_TIME_1_CYCLE;
  hqspi.Init.ClockMode = QSPI_CLOCK_MODE_0;
  hqspi.Init.FlashID = QSPI_FLASH_ID_1;
  hqspi.Init.DualFlash = QSPI_DUALFLASH_DISABLE;
  if (HAL_QSPI_Init(&hqspi) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }
}

void HAL_QSPI_MspInit(QSPI_HandleTypeDef* hqspi)
{

  GPIO_InitTypeDef GPIO_InitStruct;
  if(hqspi->Instance==QUADSPI)
  {
  /* USER CODE BEGIN QUADSPI_MspInit 0 */

  /* USER CODE END QUADSPI_MspInit 0 */
    /* Peripheral clock enable */
    __HAL_RCC_QSPI_CLK_ENABLE();
  
    /**QUADSPI GPIO Configuration    
    PF6     ------> QUADSPI_BK1_IO3
    PF7     ------> QUADSPI_BK1_IO2
    PF8     ------> QUADSPI_BK1_IO0
    PF9     ------> QUADSPI_BK1_IO1
    PF10     ------> QUADSPI_CLK
    PE11     ------> QUADSPI_BK1_NCS 
    */
    GPIO_InitStruct.Pin = GPIO_PIN_6|GPIO_PIN_7;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF10_QUADSPI;
    HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

    GPIO_InitStruct.Pin = GPIO_PIN_8|GPIO_PIN_9;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF10_QUADSPI;
    HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
	
    GPIO_InitStruct.Pin = GPIO_PIN_10;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF3_QUADSPI;
    HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

    GPIO_InitStruct.Pin = GPIO_PIN_11;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF10_QUADSPI;
    HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  /* USER CODE BEGIN QUADSPI_MspInit 1 */

  /* USER CODE END QUADSPI_MspInit 1 */
  }

}

void HAL_QSPI_MspDeInit(QSPI_HandleTypeDef* hqspi)
{

  if(hqspi->Instance==QUADSPI)
  {
  /* USER CODE BEGIN QUADSPI_MspDeInit 0 */

  /* USER CODE END QUADSPI_MspDeInit 0 */
    /* Peripheral clock disable */
    __HAL_RCC_QSPI_CLK_DISABLE();
  
    /**QUADSPI GPIO Configuration    
    PF6     ------> QUADSPI_BK1_IO3
    PF7     ------> QUADSPI_BK1_IO2
    PF8     ------> QUADSPI_BK1_IO0
    PF9     ------> QUADSPI_BK1_IO1
    PF10     ------> QUADSPI_CLK
    PE11     ------> QUADSPI_BK1_NCS 
    */
    HAL_GPIO_DeInit(GPIOF, GPIO_PIN_6|GPIO_PIN_7|GPIO_PIN_8|GPIO_PIN_9 
                          |GPIO_PIN_10);

    HAL_GPIO_DeInit(GPIOE, GPIO_PIN_11);

  /* USER CODE BEGIN QUADSPI_MspDeInit 1 */

  /* USER CODE END QUADSPI_MspDeInit 1 */
  }

}
