// Functions for easy use of QSPI memory on Kamami KAMeLeon STM32L496 board.
// jk 12'2017

#ifndef	_KAMAMI_L496_QSPI_H
#define	_KAMAMI_L496_QSPI_H
#include <stdint.h>

// The library code works with WINBOND W25Q80DVNIG memory

/* Size of single page of the QSPI memory */
#define QSPI_PAGE_SIZE                       256

#define QSPI_FLASH_SIZE                      19
/* End address of the QSPI memory */
#define QSPI_END_ADDR              					(1 << QSPI_FLASH_SIZE)

/* Initialize QUAD SPI memory. 
 * The function should be called before using any other QSPI function. 
 * It initializes GPIO lines, QSPI interface and enables quad mode of the QSPI memory. */
void qspi_init(void);

/* Write data to QSPI memeory. 
 * The memory area that gets written to, should be clean (in erased state of all 1s). 
 * @param address: memory address to write data to
 * @param buffer:  data to program
 * @param length:  number of bytes to program
 */
void qspi_write(uint32_t address, uint8_t* buffer, uint32_t length);

/* Read data from QSPI memeory. 
 * @param address: memory address to read data from
 * @param buffer:  buffer for data read from memory
 * @param length:  number of bytes to read
 */
void qspi_read(uint32_t address, uint8_t* buffer, uint32_t length);

/* Erease (clear to all 1s) 4KiB sector of memory.
 * @param address: memory address within the sector to erase
 */
void qspi_erase_sector(uint32_t address);


/* Erease (clear to all 1s) the whole memory. */
void qspi_erase_chip(void);

#endif
