// Functions for easy use of USB VCOM on Kamami KAMeLeon STM32L496 board.
// Works with STM32CubeMX generated USB CDC code.
// jk 11'2017

#include "kamami_l496_vcom.h"
#include "stm32l4xx_hal.h"
#include "string.h"
#include "usbd_cdc_if.h"

/* USB VCOM code uses CAN2_SCE interrupt to implement sending data over USB, 
 * which needs to be done at the priotity level of the USB interrupt. 
 * If CAN2_SCE is needed in your project, change the declaration below to an interrupt
 * that you don't use in your project. */
#define CDC_TX_IRQHandler	CAN2_SCE_IRQHandler
#define CDC_TX_IRQn			CAN2_SCE_IRQn

#define	CDC_BUFLEN			1024

volatile _Bool cdc_is_sending = 0;
uint8_t cdc_buf[CDC_BUFLEN];
uint32_t cdc_len = 0;

void vcom_init(void)
{
	NVIC_EnableIRQ(CDC_TX_IRQn);
	NVIC_SetPriority(CDC_TX_IRQn, 12);	
}

void vcom_send(uint8_t* buf, uint32_t len)
{
	while(len > 0)
	{
		while(cdc_is_sending);
		if(len > CDC_BUFLEN)
		{
			memcpy(cdc_buf, buf, CDC_BUFLEN);
			len -= CDC_BUFLEN;
			buf = buf + CDC_BUFLEN;
			cdc_len = CDC_BUFLEN;
			cdc_is_sending = 1;
			NVIC_SetPendingIRQ(CDC_TX_IRQn);
			
		}
		else
		{
			memcpy(cdc_buf, buf, len);
			cdc_len = len;
			cdc_is_sending = 1;
			NVIC_SetPendingIRQ(CDC_TX_IRQn);
			len = 0;
		}
	}
	
}

void CDC_TX_IRQHandler(void)
{
	CDC_Transmit_FS(cdc_buf, cdc_len);
}
